from typing import Any, Dict, List, Type, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

T = TypeVar("T", bound="WebRTCUserStateSessionsItem")


@_attrs_define
class WebRTCUserStateSessionsItem:
    """
    Attributes:
        state (str): 状態
        session_id (str): セッションID
    """

    state: str
    session_id: str
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        state = self.state

        session_id = self.session_id

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "state": state,
                "sessionId": session_id,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        state = d.pop("state")

        session_id = d.pop("sessionId")

        web_rtc_user_state_sessions_item = cls(
            state=state,
            session_id=session_id,
        )

        web_rtc_user_state_sessions_item.additional_properties = d
        return web_rtc_user_state_sessions_item

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.thumbnail_type import ThumbnailType
from ..types import UNSET, Unset

T = TypeVar("T", bound="ThumbnailInfo")


@_attrs_define
class ThumbnailInfo:
    """
    Attributes:
        type (ThumbnailType): サムネイル画像のタイプ
        mime (str): MIMEタイプ
        width (Union[Unset, int]): サムネイル幅
        height (Union[Unset, int]): サムネイル高さ
    """

    type: ThumbnailType
    mime: str
    width: Union[Unset, int] = UNSET
    height: Union[Unset, int] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        type = self.type.value

        mime = self.mime

        width = self.width

        height = self.height

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "type": type,
                "mime": mime,
            }
        )
        if width is not UNSET:
            field_dict["width"] = width
        if height is not UNSET:
            field_dict["height"] = height

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        type = ThumbnailType(d.pop("type"))

        mime = d.pop("mime")

        width = d.pop("width", UNSET)

        height = d.pop("height", UNSET)

        thumbnail_info = cls(
            type=type,
            mime=mime,
            width=width,
            height=height,
        )

        thumbnail_info.additional_properties = d
        return thumbnail_info

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.o_auth_2_scope import OAuth2Scope

T = TypeVar("T", bound="PostClientRequest")


@_attrs_define
class PostClientRequest:
    """OAuth2クライアント作成リクエスト

    Attributes:
        name (str): クライアント名
        callback_url (str): コールバックURL
        scopes (List[OAuth2Scope]): 要求スコープの配列
        description (str): 説明
    """

    name: str
    callback_url: str
    scopes: List[OAuth2Scope]
    description: str
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        name = self.name

        callback_url = self.callback_url

        scopes = []
        for scopes_item_data in self.scopes:
            scopes_item = scopes_item_data.value
            scopes.append(scopes_item)

        description = self.description

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "name": name,
                "callbackUrl": callback_url,
                "scopes": scopes,
                "description": description,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        name = d.pop("name")

        callback_url = d.pop("callbackUrl")

        scopes = []
        _scopes = d.pop("scopes")
        for scopes_item_data in _scopes:
            scopes_item = OAuth2Scope(scopes_item_data)

            scopes.append(scopes_item)

        description = d.pop("description")

        post_client_request = cls(
            name=name,
            callback_url=callback_url,
            scopes=scopes,
            description=description,
        )

        post_client_request.additional_properties = d
        return post_client_request

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

if TYPE_CHECKING:
    from ..models.user_stats_stamp import UserStatsStamp


T = TypeVar("T", bound="UserStats")


@_attrs_define
class UserStats:
    """ユーザー統計情報

    Attributes:
        total_message_count (int): ユーザーの総投稿メッセージ数(削除されたものも含む)
        stamps (List['UserStatsStamp']): ユーザーのスタンプ統計情報
        datetime_ (datetime.datetime): 統計情報日時
    """

    total_message_count: int
    stamps: List["UserStatsStamp"]
    datetime_: datetime.datetime
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        total_message_count = self.total_message_count

        stamps = []
        for stamps_item_data in self.stamps:
            stamps_item = stamps_item_data.to_dict()
            stamps.append(stamps_item)

        datetime_ = self.datetime_.isoformat()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "totalMessageCount": total_message_count,
                "stamps": stamps,
                "datetime": datetime_,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.user_stats_stamp import UserStatsStamp

        d = src_dict.copy()
        total_message_count = d.pop("totalMessageCount")

        stamps = []
        _stamps = d.pop("stamps")
        for stamps_item_data in _stamps:
            stamps_item = UserStatsStamp.from_dict(stamps_item_data)

            stamps.append(stamps_item)

        datetime_ = isoparse(d.pop("datetime"))

        user_stats = cls(
            total_message_count=total_message_count,
            stamps=stamps,
            datetime_=datetime_,
        )

        user_stats.additional_properties = d
        return user_stats

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any, Dict, Optional, Union

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.post_unlink_external_account import PostUnlinkExternalAccount
from ...types import Response


def _get_kwargs(
    *,
    body: PostUnlinkExternalAccount,
) -> Dict[str, Any]:
    headers: Dict[str, Any] = {}

    _kwargs: Dict[str, Any] = {
        "method": "post",
        "url": "/users/me/ex-accounts/unlink",
    }

    _body = body.to_dict()

    _kwargs["json"] = _body
    headers["Content-Type"] = "application/json"

    _kwargs["headers"] = headers
    return _kwargs


def _parse_response(*, client: Union[AuthenticatedClient, Client], response: httpx.Response) -> Optional[Any]:
    if response.status_code == HTTPStatus.NO_CONTENT:
        return None
    if response.status_code == HTTPStatus.BAD_REQUEST:
        return None
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(*, client: Union[AuthenticatedClient, Client], response: httpx.Response) -> Response[Any]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: Union[AuthenticatedClient, Client],
    body: PostUnlinkExternalAccount,
) -> Response[Any]:
    """外部ログインアカウントの紐付けを解除

     自分に紐付けられている外部ログインアカウントの紐付けを解除します。

    Args:
        body (PostUnlinkExternalAccount): POST /users/me/ex-accounts/unlink 用リクエストボディ

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Any]
    """

    kwargs = _get_kwargs(
        body=body,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


async def asyncio_detailed(
    *,
    client: Union[AuthenticatedClient, Client],
    body: PostUnlinkExternalAccount,
) -> Response[Any]:
    """外部ログインアカウントの紐付けを解除

     自分に紐付けられている外部ログインアカウントの紐付けを解除します。

    Args:
        body (PostUnlinkExternalAccount): POST /users/me/ex-accounts/unlink 用リクエストボディ

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Any]
    """

    kwargs = _get_kwargs(
        body=body,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)



import datetime
from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..models.channel_event_type import ChannelEventType

if TYPE_CHECKING:
    from ..models.child_created_event import ChildCreatedEvent
    from ..models.forced_notification_changed_event import ForcedNotificationChangedEvent
    from ..models.name_changed_event import NameChangedEvent
    from ..models.parent_changed_event import ParentChangedEvent
    from ..models.pin_added_event import PinAddedEvent
    from ..models.pin_removed_event import PinRemovedEvent
    from ..models.subscribers_changed_event import SubscribersChangedEvent
    from ..models.topic_changed_event import TopicChangedEvent
    from ..models.visibility_changed_event import VisibilityChangedEvent


T = TypeVar("T", bound="ChannelEvent")


@_attrs_define
class ChannelEvent:
    """チャンネルイベント

    Attributes:
        type (ChannelEventType): イベントタイプ
        datetime_ (datetime.datetime): イベント日時
        detail (Union['ChildCreatedEvent', 'ForcedNotificationChangedEvent', 'NameChangedEvent', 'ParentChangedEvent',
            'PinAddedEvent', 'PinRemovedEvent', 'SubscribersChangedEvent', 'TopicChangedEvent', 'VisibilityChangedEvent']):
            イベント内容
    """

    type: ChannelEventType
    datetime_: datetime.datetime
    detail: Union[
        "ChildCreatedEvent",
        "ForcedNotificationChangedEvent",
        "NameChangedEvent",
        "ParentChangedEvent",
        "PinAddedEvent",
        "PinRemovedEvent",
        "SubscribersChangedEvent",
        "TopicChangedEvent",
        "VisibilityChangedEvent",
    ]
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        from ..models.forced_notification_changed_event import ForcedNotificationChangedEvent
        from ..models.name_changed_event import NameChangedEvent
        from ..models.parent_changed_event import ParentChangedEvent
        from ..models.pin_added_event import PinAddedEvent
        from ..models.pin_removed_event import PinRemovedEvent
        from ..models.subscribers_changed_event import SubscribersChangedEvent
        from ..models.topic_changed_event import TopicChangedEvent
        from ..models.visibility_changed_event import VisibilityChangedEvent

        type = self.type.value

        datetime_ = self.datetime_.isoformat()

        detail: Dict[str, Any]
        if isinstance(self.detail, TopicChangedEvent):
            detail = self.detail.to_dict()
        elif isinstance(self.detail, SubscribersChangedEvent):
            detail = self.detail.to_dict()
        elif isinstance(self.detail, PinAddedEvent):
            detail = self.detail.to_dict()
        elif isinstance(self.detail, PinRemovedEvent):
            detail = self.detail.to_dict()
        elif isinstance(self.detail, NameChangedEvent):
            detail = self.detail.to_dict()
        elif isinstance(self.detail, ParentChangedEvent):
            detail = self.detail.to_dict()
        elif isinstance(self.detail, VisibilityChangedEvent):
            detail = self.detail.to_dict()
        elif isinstance(self.detail, ForcedNotificationChangedEvent):
            detail = self.detail.to_dict()
        else:
            detail = self.detail.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "type": type,
                "datetime": datetime_,
                "detail": detail,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.child_created_event import ChildCreatedEvent
        from ..models.forced_notification_changed_event import ForcedNotificationChangedEvent
        from ..models.name_changed_event import NameChangedEvent
        from ..models.parent_changed_event import ParentChangedEvent
        from ..models.pin_added_event import PinAddedEvent
        from ..models.pin_removed_event import PinRemovedEvent
        from ..models.subscribers_changed_event import SubscribersChangedEvent
        from ..models.topic_changed_event import TopicChangedEvent
        from ..models.visibility_changed_event import VisibilityChangedEvent

        d = src_dict.copy()
        type = ChannelEventType(d.pop("type"))

        datetime_ = isoparse(d.pop("datetime"))

        def _parse_detail(
            data: object,
        ) -> Union[
            "ChildCreatedEvent",
            "ForcedNotificationChangedEvent",
            "NameChangedEvent",
            "ParentChangedEvent",
            "PinAddedEvent",
            "PinRemovedEvent",
            "SubscribersChangedEvent",
            "TopicChangedEvent",
            "VisibilityChangedEvent",
        ]:
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                detail_type_0 = TopicChangedEvent.from_dict(data)

                return detail_type_0
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                detail_type_1 = SubscribersChangedEvent.from_dict(data)

                return detail_type_1
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                detail_type_2 = PinAddedEvent.from_dict(data)

                return detail_type_2
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                detail_type_3 = PinRemovedEvent.from_dict(data)

                return detail_type_3
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                detail_type_4 = NameChangedEvent.from_dict(data)

                return detail_type_4
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                detail_type_5 = ParentChangedEvent.from_dict(data)

                return detail_type_5
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                detail_type_6 = VisibilityChangedEvent.from_dict(data)

                return detail_type_6
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                detail_type_7 = ForcedNotificationChangedEvent.from_dict(data)

                return detail_type_7
            except:  # noqa: E722
                pass
            if not isinstance(data, dict):
                raise TypeError()
            detail_type_8 = ChildCreatedEvent.from_dict(data)

            return detail_type_8

        detail = _parse_detail(d.pop("detail"))

        channel_event = cls(
            type=type,
            datetime_=datetime_,
            detail=detail,
        )

        channel_event.additional_properties = d
        return channel_event

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import Any, Dict, List, Type, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..models.o_auth_2_scope import OAuth2Scope

T = TypeVar("T", bound="ActiveOAuth2Token")


@_attrs_define
class ActiveOAuth2Token:
    """有効なOAuth2トークン情報

    Attributes:
        id (str): トークンUUID
        client_id (str): OAuth2クライアントUUID
        scopes (List[OAuth2Scope]): スコープ
        issued_at (datetime.datetime): 発行日時
    """

    id: str
    client_id: str
    scopes: List[OAuth2Scope]
    issued_at: datetime.datetime
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        client_id = self.client_id

        scopes = []
        for scopes_item_data in self.scopes:
            scopes_item = scopes_item_data.value
            scopes.append(scopes_item)

        issued_at = self.issued_at.isoformat()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
                "clientId": client_id,
                "scopes": scopes,
                "issuedAt": issued_at,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id")

        client_id = d.pop("clientId")

        scopes = []
        _scopes = d.pop("scopes")
        for scopes_item_data in _scopes:
            scopes_item = OAuth2Scope(scopes_item_data)

            scopes.append(scopes_item)

        issued_at = isoparse(d.pop("issuedAt"))

        active_o_auth_2_token = cls(
            id=id,
            client_id=client_id,
            scopes=scopes,
            issued_at=issued_at,
        )

        active_o_auth_2_token.additional_properties = d
        return active_o_auth_2_token

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import Any, Dict, List, Type, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..models.user_account_state import UserAccountState

T = TypeVar("T", bound="User")


@_attrs_define
class User:
    """ユーザー情報

    Attributes:
        id (str): ユーザーUUID
        name (str): ユーザー名
        display_name (str): ユーザー表示名
        icon_file_id (str): アイコンファイルUUID
        bot (bool): BOTかどうか
        state (UserAccountState): ユーザーアカウント状態
            0: 停止
            1: 有効
            2: 一時停止
        updated_at (datetime.datetime): 更新日時
    """

    id: str
    name: str
    display_name: str
    icon_file_id: str
    bot: bool
    state: UserAccountState
    updated_at: datetime.datetime
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        name = self.name

        display_name = self.display_name

        icon_file_id = self.icon_file_id

        bot = self.bot

        state = self.state.value

        updated_at = self.updated_at.isoformat()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
                "name": name,
                "displayName": display_name,
                "iconFileId": icon_file_id,
                "bot": bot,
                "state": state,
                "updatedAt": updated_at,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id")

        name = d.pop("name")

        display_name = d.pop("displayName")

        icon_file_id = d.pop("iconFileId")

        bot = d.pop("bot")

        state = UserAccountState(d.pop("state"))

        updated_at = isoparse(d.pop("updatedAt"))

        user = cls(
            id=id,
            name=name,
            display_name=display_name,
            icon_file_id=icon_file_id,
            bot=bot,
            state=state,
            updated_at=updated_at,
        )

        user.additional_properties = d
        return user

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

if TYPE_CHECKING:
    from ..models.web_rtc_user_state_sessions_item import WebRTCUserStateSessionsItem


T = TypeVar("T", bound="WebRTCUserState")


@_attrs_define
class WebRTCUserState:
    """WebRTC状態

    Attributes:
        user_id (str): ユーザーUUID
        channel_id (str): チャンネルUUID
        sessions (List['WebRTCUserStateSessionsItem']): セッションの配列
    """

    user_id: str
    channel_id: str
    sessions: List["WebRTCUserStateSessionsItem"]
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        user_id = self.user_id

        channel_id = self.channel_id

        sessions = []
        for sessions_item_data in self.sessions:
            sessions_item = sessions_item_data.to_dict()
            sessions.append(sessions_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "userId": user_id,
                "channelId": channel_id,
                "sessions": sessions,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.web_rtc_user_state_sessions_item import WebRTCUserStateSessionsItem

        d = src_dict.copy()
        user_id = d.pop("userId")

        channel_id = d.pop("channelId")

        sessions = []
        _sessions = d.pop("sessions")
        for sessions_item_data in _sessions:
            sessions_item = WebRTCUserStateSessionsItem.from_dict(sessions_item_data)

            sessions.append(sessions_item)

        web_rtc_user_state = cls(
            user_id=user_id,
            channel_id=channel_id,
            sessions=sessions,
        )

        web_rtc_user_state.additional_properties = d
        return web_rtc_user_state

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

if TYPE_CHECKING:
    from ..models.user_group_member import UserGroupMember


T = TypeVar("T", bound="UserGroup")


@_attrs_define
class UserGroup:
    """ユーザーグループ

    Attributes:
        id (str): グループUUID
        name (str): グループ名
        description (str): グループ説明
        type (str): グループタイプ
        icon (str): グループアイコンUUID
        members (List['UserGroupMember']): グループメンバーの配列
        created_at (datetime.datetime): 作成日時
        updated_at (datetime.datetime): 更新日時
        admins (List[str]): グループ管理者のUUIDの配列
    """

    id: str
    name: str
    description: str
    type: str
    icon: str
    members: List["UserGroupMember"]
    created_at: datetime.datetime
    updated_at: datetime.datetime
    admins: List[str]
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        name = self.name

        description = self.description

        type = self.type

        icon = self.icon

        members = []
        for members_item_data in self.members:
            members_item = members_item_data.to_dict()
            members.append(members_item)

        created_at = self.created_at.isoformat()

        updated_at = self.updated_at.isoformat()

        admins = self.admins

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
                "name": name,
                "description": description,
                "type": type,
                "icon": icon,
                "members": members,
                "createdAt": created_at,
                "updatedAt": updated_at,
                "admins": admins,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.user_group_member import UserGroupMember

        d = src_dict.copy()
        id = d.pop("id")

        name = d.pop("name")

        description = d.pop("description")

        type = d.pop("type")

        icon = d.pop("icon")

        members = []
        _members = d.pop("members")
        for members_item_data in _members:
            members_item = UserGroupMember.from_dict(members_item_data)

            members.append(members_item)

        created_at = isoparse(d.pop("createdAt"))

        updated_at = isoparse(d.pop("updatedAt"))

        admins = cast(List[str], d.pop("admins"))

        user_group = cls(
            id=id,
            name=name,
            description=description,
            type=type,
            icon=icon,
            members=members,
            created_at=created_at,
            updated_at=updated_at,
            admins=admins,
        )

        user_group.additional_properties = d
        return user_group

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import Any, Dict, List, Type, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

T = TypeVar("T", bound="Webhook")


@_attrs_define
class Webhook:
    """Webhook情報

    Attributes:
        id (str): WebhookUUID
        bot_user_id (str): WebhookユーザーUUID
        display_name (str): Webhookユーザー表示名
        description (str): 説明
        secure (bool): セキュアWebhookかどうか
        channel_id (str): デフォルトの投稿先チャンネルUUID
        owner_id (str): オーナーUUID
        created_at (datetime.datetime): 作成日時
        updated_at (datetime.datetime): 更新日時
    """

    id: str
    bot_user_id: str
    display_name: str
    description: str
    secure: bool
    channel_id: str
    owner_id: str
    created_at: datetime.datetime
    updated_at: datetime.datetime
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        bot_user_id = self.bot_user_id

        display_name = self.display_name

        description = self.description

        secure = self.secure

        channel_id = self.channel_id

        owner_id = self.owner_id

        created_at = self.created_at.isoformat()

        updated_at = self.updated_at.isoformat()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
                "botUserId": bot_user_id,
                "displayName": display_name,
                "description": description,
                "secure": secure,
                "channelId": channel_id,
                "ownerId": owner_id,
                "createdAt": created_at,
                "updatedAt": updated_at,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id")

        bot_user_id = d.pop("botUserId")

        display_name = d.pop("displayName")

        description = d.pop("description")

        secure = d.pop("secure")

        channel_id = d.pop("channelId")

        owner_id = d.pop("ownerId")

        created_at = isoparse(d.pop("createdAt"))

        updated_at = isoparse(d.pop("updatedAt"))

        webhook = cls(
            id=id,
            bot_user_id=bot_user_id,
            display_name=display_name,
            description=description,
            secure=secure,
            channel_id=channel_id,
            owner_id=owner_id,
            created_at=created_at,
            updated_at=updated_at,
        )

        webhook.additional_properties = d
        return webhook

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any, Dict, Optional, Union

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...types import Response


def _get_kwargs() -> Dict[str, Any]:
    _kwargs: Dict[str, Any] = {
        "method": "get",
        "url": "/ws",
    }

    return _kwargs


def _parse_response(*, client: Union[AuthenticatedClient, Client], response: httpx.Response) -> Optional[Any]:
    if response.status_code == HTTPStatus.SWITCHING_PROTOCOLS:
        return None
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(*, client: Union[AuthenticatedClient, Client], response: httpx.Response) -> Response[Any]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: Union[AuthenticatedClient, Client],
) -> Response[Any]:
    r"""WebSocket通知ストリームに接続します

     # WebSocketプロトコル
    ## 送信
    `コマンド:引数1:引数2:...`のような形式のTextMessageをサーバーに送信することで、このWebSocketセッションに対する設定が実行できる。
    ### `viewstate`コマンド
    このWebSocketセッションが見ているチャンネル(イベントを受け取るチャンネル)を設定する。
    現時点では1つのセッションに対して1つのチャンネルしか設定できない。

    `viewstate:{チャンネルID}:{閲覧状態}`
    + チャンネルID: 対象のチャンネルID
    + 閲覧状態: `none`, `monitoring`, `editing`

    最初の`viewstate`コマンドを送る前、または`viewstate:null`, `viewstate:`を送信した後は、このセッションはどこのチャンネルも見ていないことになる。

    ### `rtcstate`コマンド
    自分のWebRTC状態を変更する。
    他のコネクションが既に状態を保持している場合、変更することができません。

    `rtcstate:{チャンネルID}:({状態}:{セッションID})*`

    コネクションが切断された場合、自分のWebRTC状態はリセットされます。

    ### `timeline_streaming`コマンド
    全てのパブリックチャンネルの`MESSAGE_CREATED`イベントを受け取るかどうかを設定する。
    初期状態は`off`です。

    `timeline_streaming:(on|off|true|false)`

    ## 受信
    TextMessageとして各種イベントが`type`と`body`を持つJSONとして非同期に送られます。

    例:
    ```json
    {\"type\":\"USER_ONLINE\",\"body\":{\"id\":\"7dd8e07f-7f5d-4331-9176-b56a4299768b\"}}
    ```

    ## イベント一覧

    ### `USER_JOINED`
    ユーザーが新規登録された。

    対象: 全員

    + `id`: 登録されたユーザーのId

    ### `USER_UPDATED`
    ユーザーの情報が更新された。

    対象: 全員

    + `id`: 情報が更新されたユーザーのId

    ### `USER_TAGS_UPDATED`
    ユーザーのタグが更新された。

    対象: 全員

    + `id`: タグが更新されたユーザーのId
    + `tag_id`: 更新されたタグのId

    ### `USER_ICON_UPDATED`
    ユーザーのアイコンが更新された。

    対象: 全員

    + `id`: アイコンが更新されたユーザーのId

    ### `USER_WEBRTC_STATE_CHANGED`
    ユーザーのWebRTCの状態が変化した

    対象: 全員

    + `user_id`: 変更があったユーザーのId
    + `channel_id`: ユーザーの変更後の接続チャンネルのId
    + `sessions`: ユーザーの変更後の状態(配列)
      + `state`: 状態
      + `sessionId`: セッションID

    ### `USER_VIEWSTATE_CHANGED`
    ユーザーのチャンネルの閲覧状態が変化した

    対象: 変化したWSセッションを含めた、該当ユーザーのWSセッション全て

    + `view_states`: 変化したWSセッションを含めた、該当ユーザーの変更後の状態(配列)
      + `key`: WSセッションの識別子
      + `channel_id`: 閲覧しているチャンネルId
      + `state`: 閲覧状態

    ### `USER_ONLINE`
    ユーザーがオンラインになった。

    対象: 全員

    + `id`: オンラインになったユーザーのId

    ### `USER_OFFLINE`
    ユーザーがオフラインになった。

    対象: 全員

    + `id`: オフラインになったユーザーのId

    ### `USER_GROUP_CREATED`
    ユーザーグループが作成された

    対象: 全員

    + `id`: 作成されたユーザーグループのId

    ### `USER_GROUP_UPDATED`
    ユーザーグループが更新された

    対象: 全員

    + `id`: 作成されたユーザーグループのId

    ### `USER_GROUP_DELETED`
    ユーザーグループが削除された

    対象: 全員

    + `id`: 削除されたユーザーグループのId

    ### `CHANNEL_CREATED`
    チャンネルが新規作成された。

    対象: 該当チャンネルを閲覧可能な全員

    + `id`: 作成されたチャンネルのId
    + `dm_user_id`: (DMの場合のみ) DM相手のユーザーId

    ### `CHANNEL_UPDATED`
    チャンネルの情報が変更された。

    対象: 該当チャンネルを閲覧可能な全員

    + `id`: 変更があったチャンネルのId
    + `dm_user_id`: (DMの場合のみ) DM相手のユーザーId

    ### `CHANNEL_DELETED`
    チャンネルが削除された。

    対象: 該当チャンネルを閲覧可能な全員

    + `id`: 削除されたチャンネルのId
    + `dm_user_id`: (DMの場合のみ) DM相手のユーザーId

    ### `CHANNEL_STARED`
    自分がチャンネルをスターした。

    対象: 自分

    + `id`: スターしたチャンネルのId

    ### `CHANNEL_UNSTARED`
    自分がチャンネルのスターを解除した。

    対象: 自分

    + `id`: スターしたチャンネルのId

    ### `CHANNEL_VIEWERS_CHANGED`
    チャンネルの閲覧者が変化した。

    対象: 該当チャンネルを閲覧しているユーザー

    + `id`: 変化したチャンネルのId
    + `viewers`: 変化後の閲覧者(配列)
      + `userId`: ユーザーId
      + `state`: 閲覧状態
      + `updatedAt`: 閲覧状態の更新日時

    ### `CHANNEL_SUBSCRIBERS_CHANGED`
    チャンネルの購読者が変化した。

    対象: 該当チャンネルを閲覧しているユーザー

    + `id`: 変化したチャンネルのId

    ### `MESSAGE_CREATED`
    メッセージが投稿された。

    対象: 投稿チャンネルを閲覧しているユーザー・投稿チャンネルに通知をつけているユーザー・メンションを受けたユーザー

    + `id`: 投稿されたメッセージのId
    + `is_citing`: 投稿されたメッセージがWebSocketを接続しているユーザーの投稿を引用しているかどうか

    ### `MESSAGE_UPDATED`
    メッセージが更新された。

    対象: 投稿チャンネルを閲覧しているユーザー

    + `id`: 更新されたメッセージのId

    ### `MESSAGE_DELETED`
    メッセージが削除された。

    対象: 投稿チャンネルを閲覧しているユーザー

    + `id`: 削除されたメッセージのId

    ### `MESSAGE_STAMPED`
    メッセージにスタンプが押された。

    対象: 投稿チャンネルを閲覧しているユーザー

    + `message_id`: メッセージId
    + `user_id`: スタンプを押したユーザーのId
    + `stamp_id`: スタンプのId
    + `count`: そのユーザーが押した数
    + `created_at`: そのユーザーがそのスタンプをそのメッセージに最初に押した日時

    ### `MESSAGE_UNSTAMPED`
    メッセージからスタンプが外された。

    対象: 投稿チャンネルを閲覧しているユーザー

    + `message_id`: メッセージId
    + `user_id`: スタンプを押したユーザーのId
    + `stamp_id`: スタンプのId

    ### `MESSAGE_PINNED`
    メッセージがピン留めされた。

    対象: 投稿チャンネルを閲覧しているユーザー

    + `message_id`: ピンされたメッセージのID
    + `channel_id`: ピンされたメッセージのチャンネルID

    ### `MESSAGE_UNPINNED`
    ピン留めされたメッセージのピンが外された。

    対象: 投稿チャンネルを閲覧しているユーザー

    + `message_id`: ピンが外されたメッセージのID
    + `channel_id`: ピンが外されたメッセージのチャンネルID

    ### `MESSAGE_READ`
    自分があるチャンネルのメッセージを読んだ。

    対象: 自分

    + `id`: 読んだチャンネルId

    ### `STAMP_CREATED`
    スタンプが新しく追加された。

    対象: 全員

    + `id`: 作成されたスタンプのId

    ### `STAMP_UPDATED`
    スタンプが修正された。

    対象: 全員

    + `id`: 修正されたスタンプのId

    ### `STAMP_DELETED`
    スタンプが削除された。

    対象: 全員

    + `id`: 削除されたスタンプのId

    ### `STAMP_PALETTE_CREATED`
    スタンプパレットが新しく追加された。

    対象: 自分

    + `id`: 作成されたスタンプパレットのId

    ### `STAMP_PALETTE_UPDATED`
    スタンプパレットが修正された。

    対象: 自分

    + `id`: 修正されたスタンプパレットのId

    ### `STAMP_PALETTE_DELETED`
    スタンプパレットが削除された。

    対象: 自分

    + `id`: 削除されたスタンプパレットのId

    ### `CLIP_FOLDER_CREATED`
    クリップフォルダーが作成された。

    対象：自分

    + `id`: 作成されたクリップフォルダーのId

    ### `CLIP_FOLDER_UPDATED`
    クリップフォルダーが修正された。

    対象: 自分

    + `id`: 更新されたクリップフォルダーのId

    ### `CLIP_FOLDER_DELETED`
    クリップフォルダーが削除された。

    対象: 自分

    + `id`: 削除されたクリップフォルダーのId

    ### `CLIP_FOLDER_MESSAGE_DELETED`
    クリップフォルダーからメッセージが除外された。

    対象: 自分

    + `folder_id`: メッセージが除外されたクリップフォルダーのId
    + `message_id`: クリップフォルダーから除外されたメッセージのId

    ### `CLIP_FOLDER_MESSAGE_ADDED`
    クリップフォルダーにメッセージが追加された。

    対象: 自分

    + `folder_id`: メッセージが追加されたクリップフォルダーのId
    + `message_id`: クリップフォルダーに追加されたメッセージのId

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Any]
    """

    kwargs = _get_kwargs()

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


async def asyncio_detailed(
    *,
    client: Union[AuthenticatedClient, Client],
) -> Response[Any]:
    r"""WebSocket通知ストリームに接続します

     # WebSocketプロトコル
    ## 送信
    `コマンド:引数1:引数2:...`のような形式のTextMessageをサーバーに送信することで、このWebSocketセッションに対する設定が実行できる。
    ### `viewstate`コマンド
    このWebSocketセッションが見ているチャンネル(イベントを受け取るチャンネル)を設定する。
    現時点では1つのセッションに対して1つのチャンネルしか設定できない。

    `viewstate:{チャンネルID}:{閲覧状態}`
    + チャンネルID: 対象のチャンネルID
    + 閲覧状態: `none`, `monitoring`, `editing`

    最初の`viewstate`コマンドを送る前、または`viewstate:null`, `viewstate:`を送信した後は、このセッションはどこのチャンネルも見ていないことになる。

    ### `rtcstate`コマンド
    自分のWebRTC状態を変更する。
    他のコネクションが既に状態を保持している場合、変更することができません。

    `rtcstate:{チャンネルID}:({状態}:{セッションID})*`

    コネクションが切断された場合、自分のWebRTC状態はリセットされます。

    ### `timeline_streaming`コマンド
    全てのパブリックチャンネルの`MESSAGE_CREATED`イベントを受け取るかどうかを設定する。
    初期状態は`off`です。

    `timeline_streaming:(on|off|true|false)`

    ## 受信
    TextMessageとして各種イベントが`type`と`body`を持つJSONとして非同期に送られます。

    例:
    ```json
    {\"type\":\"USER_ONLINE\",\"body\":{\"id\":\"7dd8e07f-7f5d-4331-9176-b56a4299768b\"}}
    ```

    ## イベント一覧

    ### `USER_JOINED`
    ユーザーが新規登録された。

    対象: 全員

    + `id`: 登録されたユーザーのId

    ### `USER_UPDATED`
    ユーザーの情報が更新された。

    対象: 全員

    + `id`: 情報が更新されたユーザーのId

    ### `USER_TAGS_UPDATED`
    ユーザーのタグが更新された。

    対象: 全員

    + `id`: タグが更新されたユーザーのId
    + `tag_id`: 更新されたタグのId

    ### `USER_ICON_UPDATED`
    ユーザーのアイコンが更新された。

    対象: 全員

    + `id`: アイコンが更新されたユーザーのId

    ### `USER_WEBRTC_STATE_CHANGED`
    ユーザーのWebRTCの状態が変化した

    対象: 全員

    + `user_id`: 変更があったユーザーのId
    + `channel_id`: ユーザーの変更後の接続チャンネルのId
    + `sessions`: ユーザーの変更後の状態(配列)
      + `state`: 状態
      + `sessionId`: セッションID

    ### `USER_VIEWSTATE_CHANGED`
    ユーザーのチャンネルの閲覧状態が変化した

    対象: 変化したWSセッションを含めた、該当ユーザーのWSセッション全て

    + `view_states`: 変化したWSセッションを含めた、該当ユーザーの変更後の状態(配列)
      + `key`: WSセッションの識別子
      + `channel_id`: 閲覧しているチャンネルId
      + `state`: 閲覧状態

    ### `USER_ONLINE`
    ユーザーがオンラインになった。

    対象: 全員

    + `id`: オンラインになったユーザーのId

    ### `USER_OFFLINE`
    ユーザーがオフラインになった。

    対象: 全員

    + `id`: オフラインになったユーザーのId

    ### `USER_GROUP_CREATED`
    ユーザーグループが作成された

    対象: 全員

    + `id`: 作成されたユーザーグループのId

    ### `USER_GROUP_UPDATED`
    ユーザーグループが更新された

    対象: 全員

    + `id`: 作成されたユーザーグループのId

    ### `USER_GROUP_DELETED`
    ユーザーグループが削除された

    対象: 全員

    + `id`: 削除されたユーザーグループのId

    ### `CHANNEL_CREATED`
    チャンネルが新規作成された。

    対象: 該当チャンネルを閲覧可能な全員

    + `id`: 作成されたチャンネルのId
    + `dm_user_id`: (DMの場合のみ) DM相手のユーザーId

    ### `CHANNEL_UPDATED`
    チャンネルの情報が変更された。

    対象: 該当チャンネルを閲覧可能な全員

    + `id`: 変更があったチャンネルのId
    + `dm_user_id`: (DMの場合のみ) DM相手のユーザーId

    ### `CHANNEL_DELETED`
    チャンネルが削除された。

    対象: 該当チャンネルを閲覧可能な全員

    + `id`: 削除されたチャンネルのId
    + `dm_user_id`: (DMの場合のみ) DM相手のユーザーId

    ### `CHANNEL_STARED`
    自分がチャンネルをスターした。

    対象: 自分

    + `id`: スターしたチャンネルのId

    ### `CHANNEL_UNSTARED`
    自分がチャンネルのスターを解除した。

    対象: 自分

    + `id`: スターしたチャンネルのId

    ### `CHANNEL_VIEWERS_CHANGED`
    チャンネルの閲覧者が変化した。

    対象: 該当チャンネルを閲覧しているユーザー

    + `id`: 変化したチャンネルのId
    + `viewers`: 変化後の閲覧者(配列)
      + `userId`: ユーザーId
      + `state`: 閲覧状態
      + `updatedAt`: 閲覧状態の更新日時

    ### `CHANNEL_SUBSCRIBERS_CHANGED`
    チャンネルの購読者が変化した。

    対象: 該当チャンネルを閲覧しているユーザー

    + `id`: 変化したチャンネルのId

    ### `MESSAGE_CREATED`
    メッセージが投稿された。

    対象: 投稿チャンネルを閲覧しているユーザー・投稿チャンネルに通知をつけているユーザー・メンションを受けたユーザー

    + `id`: 投稿されたメッセージのId
    + `is_citing`: 投稿されたメッセージがWebSocketを接続しているユーザーの投稿を引用しているかどうか

    ### `MESSAGE_UPDATED`
    メッセージが更新された。

    対象: 投稿チャンネルを閲覧しているユーザー

    + `id`: 更新されたメッセージのId

    ### `MESSAGE_DELETED`
    メッセージが削除された。

    対象: 投稿チャンネルを閲覧しているユーザー

    + `id`: 削除されたメッセージのId

    ### `MESSAGE_STAMPED`
    メッセージにスタンプが押された。

    対象: 投稿チャンネルを閲覧しているユーザー

    + `message_id`: メッセージId
    + `user_id`: スタンプを押したユーザーのId
    + `stamp_id`: スタンプのId
    + `count`: そのユーザーが押した数
    + `created_at`: そのユーザーがそのスタンプをそのメッセージに最初に押した日時

    ### `MESSAGE_UNSTAMPED`
    メッセージからスタンプが外された。

    対象: 投稿チャンネルを閲覧しているユーザー

    + `message_id`: メッセージId
    + `user_id`: スタンプを押したユーザーのId
    + `stamp_id`: スタンプのId

    ### `MESSAGE_PINNED`
    メッセージがピン留めされた。

    対象: 投稿チャンネルを閲覧しているユーザー

    + `message_id`: ピンされたメッセージのID
    + `channel_id`: ピンされたメッセージのチャンネルID

    ### `MESSAGE_UNPINNED`
    ピン留めされたメッセージのピンが外された。

    対象: 投稿チャンネルを閲覧しているユーザー

    + `message_id`: ピンが外されたメッセージのID
    + `channel_id`: ピンが外されたメッセージのチャンネルID

    ### `MESSAGE_READ`
    自分があるチャンネルのメッセージを読んだ。

    対象: 自分

    + `id`: 読んだチャンネルId

    ### `STAMP_CREATED`
    スタンプが新しく追加された。

    対象: 全員

    + `id`: 作成されたスタンプのId

    ### `STAMP_UPDATED`
    スタンプが修正された。

    対象: 全員

    + `id`: 修正されたスタンプのId

    ### `STAMP_DELETED`
    スタンプが削除された。

    対象: 全員

    + `id`: 削除されたスタンプのId

    ### `STAMP_PALETTE_CREATED`
    スタンプパレットが新しく追加された。

    対象: 自分

    + `id`: 作成されたスタンプパレットのId

    ### `STAMP_PALETTE_UPDATED`
    スタンプパレットが修正された。

    対象: 自分

    + `id`: 修正されたスタンプパレットのId

    ### `STAMP_PALETTE_DELETED`
    スタンプパレットが削除された。

    対象: 自分

    + `id`: 削除されたスタンプパレットのId

    ### `CLIP_FOLDER_CREATED`
    クリップフォルダーが作成された。

    対象：自分

    + `id`: 作成されたクリップフォルダーのId

    ### `CLIP_FOLDER_UPDATED`
    クリップフォルダーが修正された。

    対象: 自分

    + `id`: 更新されたクリップフォルダーのId

    ### `CLIP_FOLDER_DELETED`
    クリップフォルダーが削除された。

    対象: 自分

    + `id`: 削除されたクリップフォルダーのId

    ### `CLIP_FOLDER_MESSAGE_DELETED`
    クリップフォルダーからメッセージが除外された。

    対象: 自分

    + `folder_id`: メッセージが除外されたクリップフォルダーのId
    + `message_id`: クリップフォルダーから除外されたメッセージのId

    ### `CLIP_FOLDER_MESSAGE_ADDED`
    クリップフォルダーにメッセージが追加された。

    対象: 自分

    + `folder_id`: メッセージが追加されたクリップフォルダーのId
    + `message_id`: クリップフォルダーに追加されたメッセージのId

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Any]
    """

    kwargs = _get_kwargs()

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)



from typing import Any, Dict, List, Type, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

T = TypeVar("T", bound="ExternalProviderUser")


@_attrs_define
class ExternalProviderUser:
    """外部認証アカウントユーザー

    Attributes:
        provider_name (str): 外部サービス名
        linked_at (str): 紐付けた日時
        external_name (str): 外部アカウント名
    """

    provider_name: str
    linked_at: str
    external_name: str
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        provider_name = self.provider_name

        linked_at = self.linked_at

        external_name = self.external_name

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "providerName": provider_name,
                "linkedAt": linked_at,
                "externalName": external_name,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        provider_name = d.pop("providerName")

        linked_at = d.pop("linkedAt")

        external_name = d.pop("externalName")

        external_provider_user = cls(
            provider_name=provider_name,
            linked_at=linked_at,
            external_name=external_name,
        )

        external_provider_user.additional_properties = d
        return external_provider_user

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="PostMessageRequest")


@_attrs_define
class PostMessageRequest:
    """メッセージ投稿リクエスト

    Attributes:
        content (str): メッセージ本文
        embed (Union[Unset, bool]): メンション・チャンネルリンクを自動埋め込みするか Default: False.
    """

    content: str
    embed: Union[Unset, bool] = False
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        content = self.content

        embed = self.embed

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "content": content,
            }
        )
        if embed is not UNSET:
            field_dict["embed"] = embed

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        content = d.pop("content")

        embed = d.pop("embed", UNSET)

        post_message_request = cls(
            content=content,
            embed=embed,
        )

        post_message_request.additional_properties = d
        return post_message_request

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class UserPermission(str, Enum):
    ACCESS_OTHERS_BOT = "access_others_bot"
    ACCESS_OTHERS_WEBHOOK = "access_others_webhook"
    ADD_MESSAGE_STAMP = "add_message_stamp"
    BOT_ACTION_JOIN_CHANNEL = "bot_action_join_channel"
    BOT_ACTION_LEAVE_CHANNEL = "bot_action_leave_channel"
    CHANGE_MY_ICON = "change_my_icon"
    CHANGE_MY_PASSWORD = "change_my_password"
    CHANGE_PARENT_CHANNEL = "change_parent_channel"
    CONNECT_NOTIFICATION_STREAM = "connect_notification_stream"
    CREATE_BOT = "create_bot"
    CREATE_CHANNEL = "create_channel"
    CREATE_CLIENT = "create_client"
    CREATE_CLIP_FOLDER = "create_clip_folder"
    CREATE_MESSAGE_PIN = "create_message_pin"
    CREATE_SPECIAL_USER_GROUP = "create_special_user_group"
    CREATE_STAMP = "create_stamp"
    CREATE_STAMP_PALETTE = "create_stamp_palette"
    CREATE_USER_GROUP = "create_user_group"
    CREATE_WEBHOOK = "create_webhook"
    DELETE_BOT = "delete_bot"
    DELETE_CHANNEL = "delete_channel"
    DELETE_CLIP_FOLDER = "delete_clip_folder"
    DELETE_FILE = "delete_file"
    DELETE_MESSAGE = "delete_message"
    DELETE_MESSAGE_PIN = "delete_message_pin"
    DELETE_MY_CLIENT = "delete_my_client"
    DELETE_MY_SESSIONS = "delete_my_sessions"
    DELETE_STAMP = "delete_stamp"
    DELETE_STAMP_PALETTE = "delete_stamp_palette"
    DELETE_UNREAD = "delete_unread"
    DELETE_USER_GROUP = "delete_user_group"
    DELETE_WEBHOOK = "delete_webhook"
    DOWNLOAD_FILE = "download_file"
    EDIT_BOT = "edit_bot"
    EDIT_CHANNEL = "edit_channel"
    EDIT_CHANNEL_STAR = "edit_channel_star"
    EDIT_CHANNEL_SUBSCRIPTION = "edit_channel_subscription"
    EDIT_CHANNEL_TOPIC = "edit_channel_topic"
    EDIT_CLIP_FOLDER = "edit_clip_folder"
    EDIT_ME = "edit_me"
    EDIT_MESSAGE = "edit_message"
    EDIT_MY_CLIENT = "edit_my_client"
    EDIT_MY_EXTERNAL_ACCOUNT = "edit_my_external_account"
    EDIT_OTHERS_USER_GROUP = "edit_others_user_group"
    EDIT_OTHER_USERS = "edit_other_users"
    EDIT_STAMP = "edit_stamp"
    EDIT_STAMP_CREATED_BY_OTHERS = "edit_stamp_created_by_others"
    EDIT_STAMP_PALETTE = "edit_stamp_palette"
    EDIT_USER_GROUP = "edit_user_group"
    EDIT_USER_TAG = "edit_user_tag"
    EDIT_WEBHOOK = "edit_webhook"
    GET_BOT = "get_bot"
    GET_CHANNEL = "get_channel"
    GET_CHANNEL_STAR = "get_channel_star"
    GET_CHANNEL_SUBSCRIPTION = "get_channel_subscription"
    GET_CLIENTS = "get_clients"
    GET_CLIP_FOLDER = "get_clip_folder"
    GET_ME = "get_me"
    GET_MESSAGE = "get_message"
    GET_MESSAGE_REPORTS = "get_message_reports"
    GET_MY_EXTERNAL_ACCOUNT = "get_my_external_account"
    GET_MY_SESSIONS = "get_my_sessions"
    GET_MY_STAMP_HISTORY = "get_my_stamp_history"
    GET_MY_TOKENS = "get_my_tokens"
    GET_OIDC_USERINFO = "get_oidc_userinfo"
    GET_STAMP = "get_stamp"
    GET_STAMP_PALETTE = "get_stamp_palette"
    GET_UNREAD = "get_unread"
    GET_USER = "get_user"
    GET_USER_GROUP = "get_user_group"
    GET_USER_QR_CODE = "get_user_qr_code"
    GET_USER_TAG = "get_user_tag"
    GET_WEBHOOK = "get_webhook"
    MANAGE_OTHERS_CLIENT = "manage_others_client"
    POST_MESSAGE = "post_message"
    REGISTER_FCM_DEVICE = "register_fcm_device"
    REGISTER_USER = "register_user"
    REMOVE_MESSAGE_STAMP = "remove_message_stamp"
    REPORT_MESSAGE = "report_message"
    REVOKE_MY_TOKEN = "revoke_my_token"
    UPLOAD_FILE = "upload_file"
    WEB_RTC = "web_rtc"

    def __str__(self) -> str:
        return str(self.value)



from http import HTTPStatus
from typing import Any, Dict, Optional, Union, cast

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.o_auth_2_client import OAuth2Client
from ...models.o_auth_2_client_detail import OAuth2ClientDetail
from ...types import UNSET, Response, Unset


def _get_kwargs(
    client_id: str,
    *,
    detail: Union[Unset, bool] = False,
) -> Dict[str, Any]:
    params: Dict[str, Any] = {}

    params["detail"] = detail

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: Dict[str, Any] = {
        "method": "get",
        "url": f"/clients/{client_id}",
        "params": params,
    }

    return _kwargs


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[Union[Any, Union["OAuth2Client", "OAuth2ClientDetail"]]]:
    if response.status_code == HTTPStatus.OK:

        def _parse_response_200(data: object) -> Union["OAuth2Client", "OAuth2ClientDetail"]:
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                response_200_type_0 = OAuth2Client.from_dict(data)

                return response_200_type_0
            except:  # noqa: E722
                pass
            if not isinstance(data, dict):
                raise TypeError()
            response_200_type_1 = OAuth2ClientDetail.from_dict(data)

            return response_200_type_1

        response_200 = _parse_response_200(response.json())

        return response_200
    if response.status_code == HTTPStatus.FORBIDDEN:
        response_403 = cast(Any, None)
        return response_403
    if response.status_code == HTTPStatus.NOT_FOUND:
        response_404 = cast(Any, None)
        return response_404
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[Union[Any, Union["OAuth2Client", "OAuth2ClientDetail"]]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    client_id: str,
    *,
    client: Union[AuthenticatedClient, Client],
    detail: Union[Unset, bool] = False,
) -> Response[Union[Any, Union["OAuth2Client", "OAuth2ClientDetail"]]]:
    """OAuth2クライアント情報を取得

     指定したOAuth2クライアントの情報を取得します。
    詳細情報の取得には対象のクライアントの管理権限が必要です。

    Args:
        client_id (str):
        detail (Union[Unset, bool]):  Default: False.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Any, Union['OAuth2Client', 'OAuth2ClientDetail']]]
    """

    kwargs = _get_kwargs(
        client_id=client_id,
        detail=detail,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    client_id: str,
    *,
    client: Union[AuthenticatedClient, Client],
    detail: Union[Unset, bool] = False,
) -> Optional[Union[Any, Union["OAuth2Client", "OAuth2ClientDetail"]]]:
    """OAuth2クライアント情報を取得

     指定したOAuth2クライアントの情報を取得します。
    詳細情報の取得には対象のクライアントの管理権限が必要です。

    Args:
        client_id (str):
        detail (Union[Unset, bool]):  Default: False.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Any, Union['OAuth2Client', 'OAuth2ClientDetail']]
    """

    return sync_detailed(
        client_id=client_id,
        client=client,
        detail=detail,
    ).parsed


async def asyncio_detailed(
    client_id: str,
    *,
    client: Union[AuthenticatedClient, Client],
    detail: Union[Unset, bool] = False,
) -> Response[Union[Any, Union["OAuth2Client", "OAuth2ClientDetail"]]]:
    """OAuth2クライアント情報を取得

     指定したOAuth2クライアントの情報を取得します。
    詳細情報の取得には対象のクライアントの管理権限が必要です。

    Args:
        client_id (str):
        detail (Union[Unset, bool]):  Default: False.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Any, Union['OAuth2Client', 'OAuth2ClientDetail']]]
    """

    kwargs = _get_kwargs(
        client_id=client_id,
        detail=detail,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    client_id: str,
    *,
    client: Union[AuthenticatedClient, Client],
    detail: Union[Unset, bool] = False,
) -> Optional[Union[Any, Union["OAuth2Client", "OAuth2ClientDetail"]]]:
    """OAuth2クライアント情報を取得

     指定したOAuth2クライアントの情報を取得します。
    詳細情報の取得には対象のクライアントの管理権限が必要です。

    Args:
        client_id (str):
        detail (Union[Unset, bool]):  Default: False.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Any, Union['OAuth2Client', 'OAuth2ClientDetail']]
    """

    return (
        await asyncio_detailed(
            client_id=client_id,
            client=client,
            detail=detail,
        )
    ).parsed



from typing import Any, Dict, List, Type, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.channel_view_state import ChannelViewState

T = TypeVar("T", bound="MyChannelViewState")


@_attrs_define
class MyChannelViewState:
    """自身のチャンネル閲覧状態

    Attributes:
        key (str): WSセッションの識別子
        channel_id (str): チャンネルUUID
        state (ChannelViewState): 閲覧状態
    """

    key: str
    channel_id: str
    state: ChannelViewState
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        key = self.key

        channel_id = self.channel_id

        state = self.state.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "key": key,
                "channelId": channel_id,
                "state": state,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        key = d.pop("key")

        channel_id = d.pop("channelId")

        state = ChannelViewState(d.pop("state"))

        my_channel_view_state = cls(
            key=key,
            channel_id=channel_id,
            state=state,
        )

        my_channel_view_state.additional_properties = d
        return my_channel_view_state

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="PatchChannelRequest")


@_attrs_define
class PatchChannelRequest:
    """チャンネル情報変更リクエスト

    Attributes:
        name (Union[Unset, str]): チャンネル名
        archived (Union[Unset, bool]): アーカイブされているかどうか
        force (Union[Unset, bool]): 強制通知チャンネルかどうか
        parent (Union[Unset, str]): 親チャンネルUUID
    """

    name: Union[Unset, str] = UNSET
    archived: Union[Unset, bool] = UNSET
    force: Union[Unset, bool] = UNSET
    parent: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        name = self.name

        archived = self.archived

        force = self.force

        parent = self.parent

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if name is not UNSET:
            field_dict["name"] = name
        if archived is not UNSET:
            field_dict["archived"] = archived
        if force is not UNSET:
            field_dict["force"] = force
        if parent is not UNSET:
            field_dict["parent"] = parent

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        name = d.pop("name", UNSET)

        archived = d.pop("archived", UNSET)

        force = d.pop("force", UNSET)

        parent = d.pop("parent", UNSET)

        patch_channel_request = cls(
            name=name,
            archived=archived,
            force=force,
            parent=parent,
        )

        patch_channel_request.additional_properties = d
        return patch_channel_request

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.bot_mode import BotMode
from ..types import UNSET, Unset

T = TypeVar("T", bound="PatchBotRequest")


@_attrs_define
class PatchBotRequest:
    """BOT情報変更リクエスト

    Attributes:
        display_name (Union[Unset, str]): BOTユーザー表示名
        description (Union[Unset, str]): BOTの説明
        privileged (Union[Unset, bool]): 特権
        mode (Union[Unset, BotMode]): BOT動作モード

            HTTP: HTTP Mode
            WebSocket: WebSocket Mode
        endpoint (Union[Unset, str]): BOTサーバーエンドポイント
        developer_id (Union[Unset, str]): 移譲先の開発者UUID
        subscribe_events (Union[Unset, List[str]]): 購読するイベント
    """

    display_name: Union[Unset, str] = UNSET
    description: Union[Unset, str] = UNSET
    privileged: Union[Unset, bool] = UNSET
    mode: Union[Unset, BotMode] = UNSET
    endpoint: Union[Unset, str] = UNSET
    developer_id: Union[Unset, str] = UNSET
    subscribe_events: Union[Unset, List[str]] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        display_name = self.display_name

        description = self.description

        privileged = self.privileged

        mode: Union[Unset, str] = UNSET
        if not isinstance(self.mode, Unset):
            mode = self.mode.value

        endpoint = self.endpoint

        developer_id = self.developer_id

        subscribe_events: Union[Unset, List[str]] = UNSET
        if not isinstance(self.subscribe_events, Unset):
            subscribe_events = self.subscribe_events

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if display_name is not UNSET:
            field_dict["displayName"] = display_name
        if description is not UNSET:
            field_dict["description"] = description
        if privileged is not UNSET:
            field_dict["privileged"] = privileged
        if mode is not UNSET:
            field_dict["mode"] = mode
        if endpoint is not UNSET:
            field_dict["endpoint"] = endpoint
        if developer_id is not UNSET:
            field_dict["developerId"] = developer_id
        if subscribe_events is not UNSET:
            field_dict["subscribeEvents"] = subscribe_events

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        display_name = d.pop("displayName", UNSET)

        description = d.pop("description", UNSET)

        privileged = d.pop("privileged", UNSET)

        _mode = d.pop("mode", UNSET)
        mode: Union[Unset, BotMode]
        if isinstance(_mode, Unset):
            mode = UNSET
        else:
            mode = BotMode(_mode)

        endpoint = d.pop("endpoint", UNSET)

        developer_id = d.pop("developerId", UNSET)

        subscribe_events = cast(List[str], d.pop("subscribeEvents", UNSET))

        patch_bot_request = cls(
            display_name=display_name,
            description=description,
            privileged=privileged,
            mode=mode,
            endpoint=endpoint,
            developer_id=developer_id,
            subscribe_events=subscribe_events,
        )

        patch_bot_request.additional_properties = d
        return patch_bot_request

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.channel_subscribe_level import ChannelSubscribeLevel

T = TypeVar("T", bound="UserSubscribeState")


@_attrs_define
class UserSubscribeState:
    """ユーザーのチャンネル購読状態

    Attributes:
        channel_id (str): チャンネルUUID
        level (ChannelSubscribeLevel): チャンネル購読レベル
            0：無し
            1：未読管理
            2：未読管理+通知
    """

    channel_id: str
    level: ChannelSubscribeLevel
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        channel_id = self.channel_id

        level = self.level.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "channelId": channel_id,
                "level": level,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        channel_id = d.pop("channelId")

        level = ChannelSubscribeLevel(d.pop("level"))

        user_subscribe_state = cls(
            channel_id=channel_id,
            level=level,
        )

        user_subscribe_state.additional_properties = d
        return user_subscribe_state

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

T = TypeVar("T", bound="Tag")


@_attrs_define
class Tag:
    """タグ情報

    Attributes:
        id (str): タグUUID
        tag (str): タグ文字列
        users (List[str]): タグがつけられているユーザーのUUID配列
    """

    id: str
    tag: str
    users: List[str]
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        tag = self.tag

        users = self.users

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
                "tag": tag,
                "users": users,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id")

        tag = d.pop("tag")

        users = cast(List[str], d.pop("users"))

        tag = cls(
            id=id,
            tag=tag,
            users=users,
        )

        tag.additional_properties = d
        return tag

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any, Dict, Optional, Union

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...types import Response


def _get_kwargs() -> Dict[str, Any]:
    _kwargs: Dict[str, Any] = {
        "method": "get",
        "url": "/bots/ws",
    }

    return _kwargs


def _parse_response(*, client: Union[AuthenticatedClient, Client], response: httpx.Response) -> Optional[Any]:
    if response.status_code == HTTPStatus.SWITCHING_PROTOCOLS:
        return None
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(*, client: Union[AuthenticatedClient, Client], response: httpx.Response) -> Response[Any]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: Union[AuthenticatedClient, Client],
) -> Response[Any]:
    r"""WebSocket Mode BOT用通知ストリームに接続します

     # BOT WebSocketプロトコル

    ## 送信

    `コマンド:引数1:引数2:...` のような形式のTextMessageをサーバーに送信することで、このWebSocketセッションに対する設定が実行できます。

    ### `rtcstate`コマンド
    自分のWebRTC状態を変更します。
    他のコネクションが既に状態を保持している場合、変更することができません。

    `rtcstate:{チャンネルID}:({状態}:{セッションID})*`

    チャンネルIDにnullもしくは空文字を指定するか、状態にnullもしくは空文字を指定した場合、WebRTC状態はリセットされます。

    `rtcstate:null`, `rtcstate:`, `rtcstate:channelId:null`, `rtcstate:channelId:`

    コネクションが切断された場合、自分のWebRTC状態はリセットされます。

    ## 受信

    TextMessageとして各種イベントが`type`、`reqId`、`body`を持つJSONとして非同期に送られます。
    `body`の内容はHTTP Modeの場合のRequest Bodyと同様です。
    例外として`ERROR`イベントは`reqId`を持ちません。

    例: PINGイベント
    `{\"type\":\"PING\",\"reqId\":\"requestId\",\"body\":{\"eventTime\":\"2019-05-
    07T04:50:48.582586882Z\"}}`

    ### `ERROR`

    コマンドの引数が不正などの理由でコマンドが受理されなかった場合に送られます。
    非同期に送られるため、必ずしもコマンドとの対応関係を確定できないことに注意してください。
    本番環境ではERRORが送られないようにすることが望ましいです。

    `{\"type\":\"ERROR\",\"body\":\"message\"}`

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Any]
    """

    kwargs = _get_kwargs()

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


async def asyncio_detailed(
    *,
    client: Union[AuthenticatedClient, Client],
) -> Response[Any]:
    r"""WebSocket Mode BOT用通知ストリームに接続します

     # BOT WebSocketプロトコル

    ## 送信

    `コマンド:引数1:引数2:...` のような形式のTextMessageをサーバーに送信することで、このWebSocketセッションに対する設定が実行できます。

    ### `rtcstate`コマンド
    自分のWebRTC状態を変更します。
    他のコネクションが既に状態を保持している場合、変更することができません。

    `rtcstate:{チャンネルID}:({状態}:{セッションID})*`

    チャンネルIDにnullもしくは空文字を指定するか、状態にnullもしくは空文字を指定した場合、WebRTC状態はリセットされます。

    `rtcstate:null`, `rtcstate:`, `rtcstate:channelId:null`, `rtcstate:channelId:`

    コネクションが切断された場合、自分のWebRTC状態はリセットされます。

    ## 受信

    TextMessageとして各種イベントが`type`、`reqId`、`body`を持つJSONとして非同期に送られます。
    `body`の内容はHTTP Modeの場合のRequest Bodyと同様です。
    例外として`ERROR`イベントは`reqId`を持ちません。

    例: PINGイベント
    `{\"type\":\"PING\",\"reqId\":\"requestId\",\"body\":{\"eventTime\":\"2019-05-
    07T04:50:48.582586882Z\"}}`

    ### `ERROR`

    コマンドの引数が不正などの理由でコマンドが受理されなかった場合に送られます。
    非同期に送られるため、必ずしもコマンドとの対応関係を確定できないことに注意してください。
    本番環境ではERRORが送られないようにすることが望ましいです。

    `{\"type\":\"ERROR\",\"body\":\"message\"}`

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Any]
    """

    kwargs = _get_kwargs()

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)



from typing import Any, Dict, List, Type, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

T = TypeVar("T", bound="VersionFlags")


@_attrs_define
class VersionFlags:
    """
    Attributes:
        external_login (List[str]): 有効な外部ログインプロバイダ
        sign_up_allowed (bool): ユーザーが自身で新規登録(POST /api/v3/users)可能か
    """

    external_login: List[str]
    sign_up_allowed: bool
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        external_login = self.external_login

        sign_up_allowed = self.sign_up_allowed

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "externalLogin": external_login,
                "signUpAllowed": sign_up_allowed,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        external_login = cast(List[str], d.pop("externalLogin"))

        sign_up_allowed = d.pop("signUpAllowed")

        version_flags = cls(
            external_login=external_login,
            sign_up_allowed=sign_up_allowed,
        )

        version_flags.additional_properties = d
        return version_flags

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

T = TypeVar("T", bound="WebRTCAuthenticateResult")


@_attrs_define
class WebRTCAuthenticateResult:
    """skyway用認証リクエストリザルト

    Attributes:
        peer_id (str): ピアID
        ttl (int): TTL
        timestamp (int): タイムスタンプ
        auth_token (str): 認証トークン
    """

    peer_id: str
    ttl: int
    timestamp: int
    auth_token: str
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        peer_id = self.peer_id

        ttl = self.ttl

        timestamp = self.timestamp

        auth_token = self.auth_token

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "peerId": peer_id,
                "ttl": ttl,
                "timestamp": timestamp,
                "authToken": auth_token,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        peer_id = d.pop("peerId")

        ttl = d.pop("ttl")

        timestamp = d.pop("timestamp")

        auth_token = d.pop("authToken")

        web_rtc_authenticate_result = cls(
            peer_id=peer_id,
            ttl=ttl,
            timestamp=timestamp,
            auth_token=auth_token,
        )

        web_rtc_authenticate_result.additional_properties = d
        return web_rtc_authenticate_result

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import Any, Dict, List, Type, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..models.bot_mode import BotMode
from ..models.bot_state import BotState

T = TypeVar("T", bound="Bot")


@_attrs_define
class Bot:
    """BOT情報

    Attributes:
        id (str): BOT UUID
        bot_user_id (str): BOTユーザーUUID
        description (str): 説明
        developer_id (str): BOT開発者UUID
        subscribe_events (List[str]): BOTが購読しているイベントの配列
        mode (BotMode): BOT動作モード

            HTTP: HTTP Mode
            WebSocket: WebSocket Mode
        state (BotState): BOT状態
            0: 停止
            1: 有効
            2: 一時停止
        created_at (datetime.datetime): 作成日時
        updated_at (datetime.datetime): 更新日時
    """

    id: str
    bot_user_id: str
    description: str
    developer_id: str
    subscribe_events: List[str]
    mode: BotMode
    state: BotState
    created_at: datetime.datetime
    updated_at: datetime.datetime
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        bot_user_id = self.bot_user_id

        description = self.description

        developer_id = self.developer_id

        subscribe_events = self.subscribe_events

        mode = self.mode.value

        state = self.state.value

        created_at = self.created_at.isoformat()

        updated_at = self.updated_at.isoformat()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
                "botUserId": bot_user_id,
                "description": description,
                "developerId": developer_id,
                "subscribeEvents": subscribe_events,
                "mode": mode,
                "state": state,
                "createdAt": created_at,
                "updatedAt": updated_at,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id")

        bot_user_id = d.pop("botUserId")

        description = d.pop("description")

        developer_id = d.pop("developerId")

        subscribe_events = cast(List[str], d.pop("subscribeEvents"))

        mode = BotMode(d.pop("mode"))

        state = BotState(d.pop("state"))

        created_at = isoparse(d.pop("createdAt"))

        updated_at = isoparse(d.pop("updatedAt"))

        bot = cls(
            id=id,
            bot_user_id=bot_user_id,
            description=description,
            developer_id=developer_id,
            subscribe_events=subscribe_events,
            mode=mode,
            state=state,
            created_at=created_at,
            updated_at=updated_at,
        )

        bot.additional_properties = d
        return bot

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

T = TypeVar("T", bound="OgpMedia")


@_attrs_define
class OgpMedia:
    """OGPに含まれる画像の情報

    Attributes:
        url (str):
        secure_url (Union[None, str]):
        type (Union[None, str]):
        width (Union[None, int]):
        height (Union[None, int]):
    """

    url: str
    secure_url: Union[None, str]
    type: Union[None, str]
    width: Union[None, int]
    height: Union[None, int]
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        url = self.url

        secure_url: Union[None, str]
        secure_url = self.secure_url

        type: Union[None, str]
        type = self.type

        width: Union[None, int]
        width = self.width

        height: Union[None, int]
        height = self.height

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "url": url,
                "secureUrl": secure_url,
                "type": type,
                "width": width,
                "height": height,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        url = d.pop("url")

        def _parse_secure_url(data: object) -> Union[None, str]:
            if data is None:
                return data
            return cast(Union[None, str], data)

        secure_url = _parse_secure_url(d.pop("secureUrl"))

        def _parse_type(data: object) -> Union[None, str]:
            if data is None:
                return data
            return cast(Union[None, str], data)

        type = _parse_type(d.pop("type"))

        def _parse_width(data: object) -> Union[None, int]:
            if data is None:
                return data
            return cast(Union[None, int], data)

        width = _parse_width(d.pop("width"))

        def _parse_height(data: object) -> Union[None, int]:
            if data is None:
                return data
            return cast(Union[None, int], data)

        height = _parse_height(d.pop("height"))

        ogp_media = cls(
            url=url,
            secure_url=secure_url,
            type=type,
            width=width,
            height=height,
        )

        ogp_media.additional_properties = d
        return ogp_media

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.user_account_state import UserAccountState
from ..types import UNSET, Unset

T = TypeVar("T", bound="PatchUserRequest")


@_attrs_define
class PatchUserRequest:
    """ユーザー情報編集リクエスト

    Attributes:
        display_name (Union[Unset, str]): 新しい表示名
        twitter_id (Union[Unset, str]): TwitterID
        state (Union[Unset, UserAccountState]): ユーザーアカウント状態
            0: 停止
            1: 有効
            2: 一時停止
        role (Union[Unset, str]): ユーザーロール
    """

    display_name: Union[Unset, str] = UNSET
    twitter_id: Union[Unset, str] = UNSET
    state: Union[Unset, UserAccountState] = UNSET
    role: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        display_name = self.display_name

        twitter_id = self.twitter_id

        state: Union[Unset, int] = UNSET
        if not isinstance(self.state, Unset):
            state = self.state.value

        role = self.role

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if display_name is not UNSET:
            field_dict["displayName"] = display_name
        if twitter_id is not UNSET:
            field_dict["twitterId"] = twitter_id
        if state is not UNSET:
            field_dict["state"] = state
        if role is not UNSET:
            field_dict["role"] = role

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        display_name = d.pop("displayName", UNSET)

        twitter_id = d.pop("twitterId", UNSET)

        _state = d.pop("state", UNSET)
        state: Union[Unset, UserAccountState]
        if isinstance(_state, Unset):
            state = UNSET
        else:
            state = UserAccountState(_state)

        role = d.pop("role", UNSET)

        patch_user_request = cls(
            display_name=display_name,
            twitter_id=twitter_id,
            state=state,
            role=role,
        )

        patch_user_request.additional_properties = d
        return patch_user_request

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import Any, Dict, List, Type, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

T = TypeVar("T", bound="StampWithThumbnail")


@_attrs_define
class StampWithThumbnail:
    """スタンプ情報とサムネイルの有無

    Attributes:
        id (str): スタンプUUID
        name (str): スタンプ名
        creator_id (str): 作成者UUID
        created_at (datetime.datetime): 作成日時
        updated_at (datetime.datetime): 更新日時
        file_id (str): ファイルUUID
        is_unicode (bool): Unicode絵文字か
        has_thumbnail (bool): サムネイルの有無
    """

    id: str
    name: str
    creator_id: str
    created_at: datetime.datetime
    updated_at: datetime.datetime
    file_id: str
    is_unicode: bool
    has_thumbnail: bool
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        name = self.name

        creator_id = self.creator_id

        created_at = self.created_at.isoformat()

        updated_at = self.updated_at.isoformat()

        file_id = self.file_id

        is_unicode = self.is_unicode

        has_thumbnail = self.has_thumbnail

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
                "name": name,
                "creatorId": creator_id,
                "createdAt": created_at,
                "updatedAt": updated_at,
                "fileId": file_id,
                "isUnicode": is_unicode,
                "hasThumbnail": has_thumbnail,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id")

        name = d.pop("name")

        creator_id = d.pop("creatorId")

        created_at = isoparse(d.pop("createdAt"))

        updated_at = isoparse(d.pop("updatedAt"))

        file_id = d.pop("fileId")

        is_unicode = d.pop("isUnicode")

        has_thumbnail = d.pop("hasThumbnail")

        stamp_with_thumbnail = cls(
            id=id,
            name=name,
            creator_id=creator_id,
            created_at=created_at,
            updated_at=updated_at,
            file_id=file_id,
            is_unicode=is_unicode,
            has_thumbnail=has_thumbnail,
        )

        stamp_with_thumbnail.additional_properties = d
        return stamp_with_thumbnail

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any, Dict, Optional, Union

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.o_auth_2_prompt import OAuth2Prompt
from ...models.o_auth_2_response_type import OAuth2ResponseType
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    response_type: Union[Unset, OAuth2ResponseType] = UNSET,
    client_id: str,
    redirect_uri: Union[Unset, str] = UNSET,
    scope: Union[Unset, str] = UNSET,
    state: Union[Unset, str] = UNSET,
    code_challenge: Union[Unset, str] = UNSET,
    code_challenge_method: Union[Unset, str] = UNSET,
    nonce: Union[Unset, str] = UNSET,
    prompt: Union[Unset, OAuth2Prompt] = UNSET,
) -> Dict[str, Any]:
    params: Dict[str, Any] = {}

    json_response_type: Union[Unset, str] = UNSET
    if not isinstance(response_type, Unset):
        json_response_type = response_type.value

    params["response_type"] = json_response_type

    params["client_id"] = client_id

    params["redirect_uri"] = redirect_uri

    params["scope"] = scope

    params["state"] = state

    params["code_challenge"] = code_challenge

    params["code_challenge_method"] = code_challenge_method

    params["nonce"] = nonce

    json_prompt: Union[Unset, str] = UNSET
    if not isinstance(prompt, Unset):
        json_prompt = prompt.value

    params["prompt"] = json_prompt

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: Dict[str, Any] = {
        "method": "get",
        "url": "/oauth2/authorize",
        "params": params,
    }

    return _kwargs


def _parse_response(*, client: Union[AuthenticatedClient, Client], response: httpx.Response) -> Optional[Any]:
    if response.status_code == HTTPStatus.FOUND:
        return None
    if response.status_code == HTTPStatus.BAD_REQUEST:
        return None
    if response.status_code == HTTPStatus.FORBIDDEN:
        return None
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(*, client: Union[AuthenticatedClient, Client], response: httpx.Response) -> Response[Any]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: Union[AuthenticatedClient, Client],
    response_type: Union[Unset, OAuth2ResponseType] = UNSET,
    client_id: str,
    redirect_uri: Union[Unset, str] = UNSET,
    scope: Union[Unset, str] = UNSET,
    state: Union[Unset, str] = UNSET,
    code_challenge: Union[Unset, str] = UNSET,
    code_challenge_method: Union[Unset, str] = UNSET,
    nonce: Union[Unset, str] = UNSET,
    prompt: Union[Unset, OAuth2Prompt] = UNSET,
) -> Response[Any]:
    """OAuth2 認可エンドポイント

     OAuth2 認可エンドポイント

    Args:
        response_type (Union[Unset, OAuth2ResponseType]):
        client_id (str):
        redirect_uri (Union[Unset, str]):
        scope (Union[Unset, str]):
        state (Union[Unset, str]):
        code_challenge (Union[Unset, str]):
        code_challenge_method (Union[Unset, str]):
        nonce (Union[Unset, str]):
        prompt (Union[Unset, OAuth2Prompt]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Any]
    """

    kwargs = _get_kwargs(
        response_type=response_type,
        client_id=client_id,
        redirect_uri=redirect_uri,
        scope=scope,
        state=state,
        code_challenge=code_challenge,
        code_challenge_method=code_challenge_method,
        nonce=nonce,
        prompt=prompt,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


async def asyncio_detailed(
    *,
    client: Union[AuthenticatedClient, Client],
    response_type: Union[Unset, OAuth2ResponseType] = UNSET,
    client_id: str,
    redirect_uri: Union[Unset, str] = UNSET,
    scope: Union[Unset, str] = UNSET,
    state: Union[Unset, str] = UNSET,
    code_challenge: Union[Unset, str] = UNSET,
    code_challenge_method: Union[Unset, str] = UNSET,
    nonce: Union[Unset, str] = UNSET,
    prompt: Union[Unset, OAuth2Prompt] = UNSET,
) -> Response[Any]:
    """OAuth2 認可エンドポイント

     OAuth2 認可エンドポイント

    Args:
        response_type (Union[Unset, OAuth2ResponseType]):
        client_id (str):
        redirect_uri (Union[Unset, str]):
        scope (Union[Unset, str]):
        state (Union[Unset, str]):
        code_challenge (Union[Unset, str]):
        code_challenge_method (Union[Unset, str]):
        nonce (Union[Unset, str]):
        prompt (Union[Unset, OAuth2Prompt]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Any]
    """

    kwargs = _get_kwargs(
        response_type=response_type,
        client_id=client_id,
        redirect_uri=redirect_uri,
        scope=scope,
        state=state,
        code_challenge=code_challenge,
        code_challenge_method=code_challenge_method,
        nonce=nonce,
        prompt=prompt,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)



import datetime
from http import HTTPStatus
from typing import Any, Dict, Optional, Union, cast

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.search_messages_message_search_result import SearchMessagesMessageSearchResult
from ...models.search_messages_sort import SearchMessagesSort
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    word: Union[Unset, str] = UNSET,
    after: Union[Unset, datetime.datetime] = UNSET,
    before: Union[Unset, datetime.datetime] = UNSET,
    in_: Union[Unset, str] = UNSET,
    to: Union[Unset, str] = UNSET,
    from_: Union[Unset, str] = UNSET,
    citation: Union[Unset, str] = UNSET,
    bot: Union[Unset, bool] = UNSET,
    has_url: Union[Unset, bool] = UNSET,
    has_attachments: Union[Unset, bool] = UNSET,
    has_image: Union[Unset, bool] = UNSET,
    has_video: Union[Unset, bool] = UNSET,
    has_audio: Union[Unset, bool] = UNSET,
    limit: Union[Unset, int] = UNSET,
    offset: Union[Unset, int] = UNSET,
    sort: Union[Unset, SearchMessagesSort] = SearchMessagesSort.VALUE_1,
) -> Dict[str, Any]:
    params: Dict[str, Any] = {}

    params["word"] = word

    json_after: Union[Unset, str] = UNSET
    if not isinstance(after, Unset):
        json_after = after.isoformat()
    params["after"] = json_after

    json_before: Union[Unset, str] = UNSET
    if not isinstance(before, Unset):
        json_before = before.isoformat()
    params["before"] = json_before

    params["in"] = in_

    params["to"] = to

    params["from"] = from_

    params["citation"] = citation

    params["bot"] = bot

    params["hasURL"] = has_url

    params["hasAttachments"] = has_attachments

    params["hasImage"] = has_image

    params["hasVideo"] = has_video

    params["hasAudio"] = has_audio

    params["limit"] = limit

    params["offset"] = offset

    json_sort: Union[Unset, str] = UNSET
    if not isinstance(sort, Unset):
        json_sort = sort.value

    params["sort"] = json_sort

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: Dict[str, Any] = {
        "method": "get",
        "url": "/messages",
        "params": params,
    }

    return _kwargs


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[Union[Any, SearchMessagesMessageSearchResult]]:
    if response.status_code == HTTPStatus.OK:
        response_200 = SearchMessagesMessageSearchResult.from_dict(response.json())

        return response_200
    if response.status_code == HTTPStatus.BAD_REQUEST:
        response_400 = cast(Any, None)
        return response_400
    if response.status_code == HTTPStatus.SERVICE_UNAVAILABLE:
        response_503 = cast(Any, None)
        return response_503
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[Union[Any, SearchMessagesMessageSearchResult]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: Union[AuthenticatedClient, Client],
    word: Union[Unset, str] = UNSET,
    after: Union[Unset, datetime.datetime] = UNSET,
    before: Union[Unset, datetime.datetime] = UNSET,
    in_: Union[Unset, str] = UNSET,
    to: Union[Unset, str] = UNSET,
    from_: Union[Unset, str] = UNSET,
    citation: Union[Unset, str] = UNSET,
    bot: Union[Unset, bool] = UNSET,
    has_url: Union[Unset, bool] = UNSET,
    has_attachments: Union[Unset, bool] = UNSET,
    has_image: Union[Unset, bool] = UNSET,
    has_video: Union[Unset, bool] = UNSET,
    has_audio: Union[Unset, bool] = UNSET,
    limit: Union[Unset, int] = UNSET,
    offset: Union[Unset, int] = UNSET,
    sort: Union[Unset, SearchMessagesSort] = SearchMessagesSort.VALUE_1,
) -> Response[Union[Any, SearchMessagesMessageSearchResult]]:
    """メッセージを検索

     メッセージを検索します。

    Args:
        word (Union[Unset, str]):
        after (Union[Unset, datetime.datetime]):
        before (Union[Unset, datetime.datetime]):
        in_ (Union[Unset, str]):
        to (Union[Unset, str]):
        from_ (Union[Unset, str]):
        citation (Union[Unset, str]):
        bot (Union[Unset, bool]):
        has_url (Union[Unset, bool]):
        has_attachments (Union[Unset, bool]):
        has_image (Union[Unset, bool]):
        has_video (Union[Unset, bool]):
        has_audio (Union[Unset, bool]):
        limit (Union[Unset, int]):
        offset (Union[Unset, int]):
        sort (Union[Unset, SearchMessagesSort]):  Default: SearchMessagesSort.VALUE_1.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Any, SearchMessagesMessageSearchResult]]
    """

    kwargs = _get_kwargs(
        word=word,
        after=after,
        before=before,
        in_=in_,
        to=to,
        from_=from_,
        citation=citation,
        bot=bot,
        has_url=has_url,
        has_attachments=has_attachments,
        has_image=has_image,
        has_video=has_video,
        has_audio=has_audio,
        limit=limit,
        offset=offset,
        sort=sort,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    *,
    client: Union[AuthenticatedClient, Client],
    word: Union[Unset, str] = UNSET,
    after: Union[Unset, datetime.datetime] = UNSET,
    before: Union[Unset, datetime.datetime] = UNSET,
    in_: Union[Unset, str] = UNSET,
    to: Union[Unset, str] = UNSET,
    from_: Union[Unset, str] = UNSET,
    citation: Union[Unset, str] = UNSET,
    bot: Union[Unset, bool] = UNSET,
    has_url: Union[Unset, bool] = UNSET,
    has_attachments: Union[Unset, bool] = UNSET,
    has_image: Union[Unset, bool] = UNSET,
    has_video: Union[Unset, bool] = UNSET,
    has_audio: Union[Unset, bool] = UNSET,
    limit: Union[Unset, int] = UNSET,
    offset: Union[Unset, int] = UNSET,
    sort: Union[Unset, SearchMessagesSort] = SearchMessagesSort.VALUE_1,
) -> Optional[Union[Any, SearchMessagesMessageSearchResult]]:
    """メッセージを検索

     メッセージを検索します。

    Args:
        word (Union[Unset, str]):
        after (Union[Unset, datetime.datetime]):
        before (Union[Unset, datetime.datetime]):
        in_ (Union[Unset, str]):
        to (Union[Unset, str]):
        from_ (Union[Unset, str]):
        citation (Union[Unset, str]):
        bot (Union[Unset, bool]):
        has_url (Union[Unset, bool]):
        has_attachments (Union[Unset, bool]):
        has_image (Union[Unset, bool]):
        has_video (Union[Unset, bool]):
        has_audio (Union[Unset, bool]):
        limit (Union[Unset, int]):
        offset (Union[Unset, int]):
        sort (Union[Unset, SearchMessagesSort]):  Default: SearchMessagesSort.VALUE_1.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Any, SearchMessagesMessageSearchResult]
    """

    return sync_detailed(
        client=client,
        word=word,
        after=after,
        before=before,
        in_=in_,
        to=to,
        from_=from_,
        citation=citation,
        bot=bot,
        has_url=has_url,
        has_attachments=has_attachments,
        has_image=has_image,
        has_video=has_video,
        has_audio=has_audio,
        limit=limit,
        offset=offset,
        sort=sort,
    ).parsed


async def asyncio_detailed(
    *,
    client: Union[AuthenticatedClient, Client],
    word: Union[Unset, str] = UNSET,
    after: Union[Unset, datetime.datetime] = UNSET,
    before: Union[Unset, datetime.datetime] = UNSET,
    in_: Union[Unset, str] = UNSET,
    to: Union[Unset, str] = UNSET,
    from_: Union[Unset, str] = UNSET,
    citation: Union[Unset, str] = UNSET,
    bot: Union[Unset, bool] = UNSET,
    has_url: Union[Unset, bool] = UNSET,
    has_attachments: Union[Unset, bool] = UNSET,
    has_image: Union[Unset, bool] = UNSET,
    has_video: Union[Unset, bool] = UNSET,
    has_audio: Union[Unset, bool] = UNSET,
    limit: Union[Unset, int] = UNSET,
    offset: Union[Unset, int] = UNSET,
    sort: Union[Unset, SearchMessagesSort] = SearchMessagesSort.VALUE_1,
) -> Response[Union[Any, SearchMessagesMessageSearchResult]]:
    """メッセージを検索

     メッセージを検索します。

    Args:
        word (Union[Unset, str]):
        after (Union[Unset, datetime.datetime]):
        before (Union[Unset, datetime.datetime]):
        in_ (Union[Unset, str]):
        to (Union[Unset, str]):
        from_ (Union[Unset, str]):
        citation (Union[Unset, str]):
        bot (Union[Unset, bool]):
        has_url (Union[Unset, bool]):
        has_attachments (Union[Unset, bool]):
        has_image (Union[Unset, bool]):
        has_video (Union[Unset, bool]):
        has_audio (Union[Unset, bool]):
        limit (Union[Unset, int]):
        offset (Union[Unset, int]):
        sort (Union[Unset, SearchMessagesSort]):  Default: SearchMessagesSort.VALUE_1.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Any, SearchMessagesMessageSearchResult]]
    """

    kwargs = _get_kwargs(
        word=word,
        after=after,
        before=before,
        in_=in_,
        to=to,
        from_=from_,
        citation=citation,
        bot=bot,
        has_url=has_url,
        has_attachments=has_attachments,
        has_image=has_image,
        has_video=has_video,
        has_audio=has_audio,
        limit=limit,
        offset=offset,
        sort=sort,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    *,
    client: Union[AuthenticatedClient, Client],
    word: Union[Unset, str] = UNSET,
    after: Union[Unset, datetime.datetime] = UNSET,
    before: Union[Unset, datetime.datetime] = UNSET,
    in_: Union[Unset, str] = UNSET,
    to: Union[Unset, str] = UNSET,
    from_: Union[Unset, str] = UNSET,
    citation: Union[Unset, str] = UNSET,
    bot: Union[Unset, bool] = UNSET,
    has_url: Union[Unset, bool] = UNSET,
    has_attachments: Union[Unset, bool] = UNSET,
    has_image: Union[Unset, bool] = UNSET,
    has_video: Union[Unset, bool] = UNSET,
    has_audio: Union[Unset, bool] = UNSET,
    limit: Union[Unset, int] = UNSET,
    offset: Union[Unset, int] = UNSET,
    sort: Union[Unset, SearchMessagesSort] = SearchMessagesSort.VALUE_1,
) -> Optional[Union[Any, SearchMessagesMessageSearchResult]]:
    """メッセージを検索

     メッセージを検索します。

    Args:
        word (Union[Unset, str]):
        after (Union[Unset, datetime.datetime]):
        before (Union[Unset, datetime.datetime]):
        in_ (Union[Unset, str]):
        to (Union[Unset, str]):
        from_ (Union[Unset, str]):
        citation (Union[Unset, str]):
        bot (Union[Unset, bool]):
        has_url (Union[Unset, bool]):
        has_attachments (Union[Unset, bool]):
        has_image (Union[Unset, bool]):
        has_video (Union[Unset, bool]):
        has_audio (Union[Unset, bool]):
        limit (Union[Unset, int]):
        offset (Union[Unset, int]):
        sort (Union[Unset, SearchMessagesSort]):  Default: SearchMessagesSort.VALUE_1.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Any, SearchMessagesMessageSearchResult]
    """

    return (
        await asyncio_detailed(
            client=client,
            word=word,
            after=after,
            before=before,
            in_=in_,
            to=to,
            from_=from_,
            citation=citation,
            bot=bot,
            has_url=has_url,
            has_attachments=has_attachments,
            has_image=has_image,
            has_video=has_video,
            has_audio=has_audio,
            limit=limit,
            offset=offset,
            sort=sort,
        )
    ).parsed



import datetime
from typing import Any, Dict, List, Type, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

T = TypeVar("T", bound="UnreadChannel")


@_attrs_define
class UnreadChannel:
    """未読チャンネル情報

    Attributes:
        channel_id (str): チャンネルUUID
        count (int): 未読メッセージ数
        noticeable (bool): 自分宛てメッセージが含まれているかどうか
        since (datetime.datetime): チャンネルの最古の未読メッセージの日時
        updated_at (datetime.datetime): チャンネルの最新の未読メッセージの日時
        oldest_message_id (str): そのチャンネルの未読の中で最も古いメッセージのid
    """

    channel_id: str
    count: int
    noticeable: bool
    since: datetime.datetime
    updated_at: datetime.datetime
    oldest_message_id: str
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        channel_id = self.channel_id

        count = self.count

        noticeable = self.noticeable

        since = self.since.isoformat()

        updated_at = self.updated_at.isoformat()

        oldest_message_id = self.oldest_message_id

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "channelId": channel_id,
                "count": count,
                "noticeable": noticeable,
                "since": since,
                "updatedAt": updated_at,
                "oldestMessageId": oldest_message_id,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        channel_id = d.pop("channelId")

        count = d.pop("count")

        noticeable = d.pop("noticeable")

        since = isoparse(d.pop("since"))

        updated_at = isoparse(d.pop("updatedAt"))

        oldest_message_id = d.pop("oldestMessageId")

        unread_channel = cls(
            channel_id=channel_id,
            count=count,
            noticeable=noticeable,
            since=since,
            updated_at=updated_at,
            oldest_message_id=oldest_message_id,
        )

        unread_channel.additional_properties = d
        return unread_channel

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class ChannelEventType(str, Enum):
    CHILDCREATED = "ChildCreated"
    FORCEDNOTIFICATIONCHANGED = "ForcedNotificationChanged"
    NAMECHANGED = "NameChanged"
    PARENTCHANGED = "ParentChanged"
    PINADDED = "PinAdded"
    PINREMOVED = "PinRemoved"
    SUBSCRIBERSCHANGED = "SubscribersChanged"
    TOPICCHANGED = "TopicChanged"
    VISIBILITYCHANGED = "VisibilityChanged"

    def __str__(self) -> str:
        return str(self.value)



from http import HTTPStatus
from typing import Any, Dict, Optional, Union, cast

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.ogp import Ogp
from ...types import UNSET, Response


def _get_kwargs(
    *,
    url_query: str,
) -> Dict[str, Any]:
    params: Dict[str, Any] = {}

    params["url"] = url_query

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: Dict[str, Any] = {
        "method": "get",
        "url": "/ogp",
        "params": params,
    }

    return _kwargs


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[Union[Any, Ogp]]:
    if response.status_code == HTTPStatus.OK:
        response_200 = Ogp.from_dict(response.json())

        return response_200
    if response.status_code == HTTPStatus.BAD_REQUEST:
        response_400 = cast(Any, None)
        return response_400
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[Union[Any, Ogp]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: Union[AuthenticatedClient, Client],
    url_query: str,
) -> Response[Union[Any, Ogp]]:
    """OGP情報を取得

     指定されたURLのOGP情報を取得します。
    指定されたURLに対するOGP情報が見つからなかった場合、typeがemptyに設定された空のOGP情報を返します。

    Args:
        url_query (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Any, Ogp]]
    """

    kwargs = _get_kwargs(
        url_query=url_query,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    *,
    client: Union[AuthenticatedClient, Client],
    url_query: str,
) -> Optional[Union[Any, Ogp]]:
    """OGP情報を取得

     指定されたURLのOGP情報を取得します。
    指定されたURLに対するOGP情報が見つからなかった場合、typeがemptyに設定された空のOGP情報を返します。

    Args:
        url_query (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Any, Ogp]
    """

    return sync_detailed(
        client=client,
        url_query=url_query,
    ).parsed


async def asyncio_detailed(
    *,
    client: Union[AuthenticatedClient, Client],
    url_query: str,
) -> Response[Union[Any, Ogp]]:
    """OGP情報を取得

     指定されたURLのOGP情報を取得します。
    指定されたURLに対するOGP情報が見つからなかった場合、typeがemptyに設定された空のOGP情報を返します。

    Args:
        url_query (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Any, Ogp]]
    """

    kwargs = _get_kwargs(
        url_query=url_query,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    *,
    client: Union[AuthenticatedClient, Client],
    url_query: str,
) -> Optional[Union[Any, Ogp]]:
    """OGP情報を取得

     指定されたURLのOGP情報を取得します。
    指定されたURLに対するOGP情報が見つからなかった場合、typeがemptyに設定された空のOGP情報を返します。

    Args:
        url_query (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Any, Ogp]
    """

    return (
        await asyncio_detailed(
            client=client,
            url_query=url_query,
        )
    ).parsed



from typing import Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="PatchChannelSubscribersRequest")


@_attrs_define
class PatchChannelSubscribersRequest:
    """チャンネル購読者編集リクエスト

    Attributes:
        on (Union[Unset, List[str]]): 通知をオンにするユーザーのUUID配列
        off (Union[Unset, List[str]]): 通知をオフにするユーザーのUUID配列
    """

    on: Union[Unset, List[str]] = UNSET
    off: Union[Unset, List[str]] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        on: Union[Unset, List[str]] = UNSET
        if not isinstance(self.on, Unset):
            on = self.on

        off: Union[Unset, List[str]] = UNSET
        if not isinstance(self.off, Unset):
            off = self.off

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if on is not UNSET:
            field_dict["on"] = on
        if off is not UNSET:
            field_dict["off"] = off

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        on = cast(List[str], d.pop("on", UNSET))

        off = cast(List[str], d.pop("off", UNSET))

        patch_channel_subscribers_request = cls(
            on=on,
            off=off,
        )

        patch_channel_subscribers_request.additional_properties = d
        return patch_channel_subscribers_request

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="PatchWebhookRequest")


@_attrs_define
class PatchWebhookRequest:
    """Webhook情報変更リクエスト

    Attributes:
        name (Union[Unset, str]): Webhookユーザー表示名
        description (Union[Unset, str]): 説明
        channel_id (Union[Unset, str]): デフォルトの投稿先チャンネルUUID
        secret (Union[Unset, str]): Webhookシークレット
        owner_id (Union[Unset, str]): 移譲先のユーザーUUID
    """

    name: Union[Unset, str] = UNSET
    description: Union[Unset, str] = UNSET
    channel_id: Union[Unset, str] = UNSET
    secret: Union[Unset, str] = UNSET
    owner_id: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        name = self.name

        description = self.description

        channel_id = self.channel_id

        secret = self.secret

        owner_id = self.owner_id

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if name is not UNSET:
            field_dict["name"] = name
        if description is not UNSET:
            field_dict["description"] = description
        if channel_id is not UNSET:
            field_dict["channelId"] = channel_id
        if secret is not UNSET:
            field_dict["secret"] = secret
        if owner_id is not UNSET:
            field_dict["ownerId"] = owner_id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        name = d.pop("name", UNSET)

        description = d.pop("description", UNSET)

        channel_id = d.pop("channelId", UNSET)

        secret = d.pop("secret", UNSET)

        owner_id = d.pop("ownerId", UNSET)

        patch_webhook_request = cls(
            name=name,
            description=description,
            channel_id=channel_id,
            secret=secret,
            owner_id=owner_id,
        )

        patch_webhook_request.additional_properties = d
        return patch_webhook_request

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="PatchStampPaletteRequest")


@_attrs_define
class PatchStampPaletteRequest:
    """スタンプパレット情報変更リクエスト

    Attributes:
        name (Union[Unset, str]): パレット名
        description (Union[Unset, str]): 説明
        stamps (Union[Unset, List[str]]): パレット内のスタンプUUIDの配列
    """

    name: Union[Unset, str] = UNSET
    description: Union[Unset, str] = UNSET
    stamps: Union[Unset, List[str]] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        name = self.name

        description = self.description

        stamps: Union[Unset, List[str]] = UNSET
        if not isinstance(self.stamps, Unset):
            stamps = self.stamps

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if name is not UNSET:
            field_dict["name"] = name
        if description is not UNSET:
            field_dict["description"] = description
        if stamps is not UNSET:
            field_dict["stamps"] = stamps

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        name = d.pop("name", UNSET)

        description = d.pop("description", UNSET)

        stamps = cast(List[str], d.pop("stamps", UNSET))

        patch_stamp_palette_request = cls(
            name=name,
            description=description,
            stamps=stamps,
        )

        patch_stamp_palette_request.additional_properties = d
        return patch_stamp_palette_request

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class SearchMessagesSort(str, Enum):
    CREATEDAT = "createdAt"
    UPDATEDAT = "updatedAt"
    VALUE_1 = "-createdAt"
    VALUE_3 = "-updatedAt"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.o_auth_2_prompt import OAuth2Prompt
from ..models.o_auth_2_response_type import OAuth2ResponseType
from ..types import UNSET, Unset

T = TypeVar("T", bound="OAuth2Authorization")


@_attrs_define
class OAuth2Authorization:
    """
    Attributes:
        client_id (str):
        response_type (Union[Unset, OAuth2ResponseType]):
        redirect_uri (Union[Unset, str]):
        scope (Union[Unset, str]):
        state (Union[Unset, str]):
        code_challenge (Union[Unset, str]):
        code_challenge_method (Union[Unset, str]):
        nonce (Union[Unset, str]):
        prompt (Union[Unset, OAuth2Prompt]):
    """

    client_id: str
    response_type: Union[Unset, OAuth2ResponseType] = UNSET
    redirect_uri: Union[Unset, str] = UNSET
    scope: Union[Unset, str] = UNSET
    state: Union[Unset, str] = UNSET
    code_challenge: Union[Unset, str] = UNSET
    code_challenge_method: Union[Unset, str] = UNSET
    nonce: Union[Unset, str] = UNSET
    prompt: Union[Unset, OAuth2Prompt] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        client_id = self.client_id

        response_type: Union[Unset, str] = UNSET
        if not isinstance(self.response_type, Unset):
            response_type = self.response_type.value

        redirect_uri = self.redirect_uri

        scope = self.scope

        state = self.state

        code_challenge = self.code_challenge

        code_challenge_method = self.code_challenge_method

        nonce = self.nonce

        prompt: Union[Unset, str] = UNSET
        if not isinstance(self.prompt, Unset):
            prompt = self.prompt.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "client_id": client_id,
            }
        )
        if response_type is not UNSET:
            field_dict["response_type"] = response_type
        if redirect_uri is not UNSET:
            field_dict["redirect_uri"] = redirect_uri
        if scope is not UNSET:
            field_dict["scope"] = scope
        if state is not UNSET:
            field_dict["state"] = state
        if code_challenge is not UNSET:
            field_dict["code_challenge"] = code_challenge
        if code_challenge_method is not UNSET:
            field_dict["code_challenge_method"] = code_challenge_method
        if nonce is not UNSET:
            field_dict["nonce"] = nonce
        if prompt is not UNSET:
            field_dict["prompt"] = prompt

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        client_id = d.pop("client_id")

        _response_type = d.pop("response_type", UNSET)
        response_type: Union[Unset, OAuth2ResponseType]
        if isinstance(_response_type, Unset):
            response_type = UNSET
        else:
            response_type = OAuth2ResponseType(_response_type)

        redirect_uri = d.pop("redirect_uri", UNSET)

        scope = d.pop("scope", UNSET)

        state = d.pop("state", UNSET)

        code_challenge = d.pop("code_challenge", UNSET)

        code_challenge_method = d.pop("code_challenge_method", UNSET)

        nonce = d.pop("nonce", UNSET)

        _prompt = d.pop("prompt", UNSET)
        prompt: Union[Unset, OAuth2Prompt]
        if isinstance(_prompt, Unset):
            prompt = UNSET
        else:
            prompt = OAuth2Prompt(_prompt)

        o_auth_2_authorization = cls(
            client_id=client_id,
            response_type=response_type,
            redirect_uri=redirect_uri,
            scope=scope,
            state=state,
            code_challenge=code_challenge,
            code_challenge_method=code_challenge_method,
            nonce=nonce,
            prompt=prompt,
        )

        o_auth_2_authorization.additional_properties = d
        return o_auth_2_authorization

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class OAuth2Scope(str, Enum):
    MANAGE_BOT = "manage_bot"
    OPENID = "openid"
    PROFILE = "profile"
    READ = "read"
    WRITE = "write"

    def __str__(self) -> str:
        return str(self.value)



import datetime
from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..models.bot_event_result import BotEventResult
from ..types import UNSET, Unset

T = TypeVar("T", bound="BotEventLog")


@_attrs_define
class BotEventLog:
    """BOTイベントログ

    Attributes:
        bot_id (str): BOT UUID
        request_id (str): リクエストUUID
        event (str): イベントタイプ
        code (int): ステータスコード
        datetime_ (datetime.datetime): イベント日時
        result (Union[Unset, BotEventResult]): イベント配送結果
    """

    bot_id: str
    request_id: str
    event: str
    code: int
    datetime_: datetime.datetime
    result: Union[Unset, BotEventResult] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        bot_id = self.bot_id

        request_id = self.request_id

        event = self.event

        code = self.code

        datetime_ = self.datetime_.isoformat()

        result: Union[Unset, str] = UNSET
        if not isinstance(self.result, Unset):
            result = self.result.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "botId": bot_id,
                "requestId": request_id,
                "event": event,
                "code": code,
                "datetime": datetime_,
            }
        )
        if result is not UNSET:
            field_dict["result"] = result

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        bot_id = d.pop("botId")

        request_id = d.pop("requestId")

        event = d.pop("event")

        code = d.pop("code")

        datetime_ = isoparse(d.pop("datetime"))

        _result = d.pop("result", UNSET)
        result: Union[Unset, BotEventResult]
        if isinstance(_result, Unset):
            result = UNSET
        else:
            result = BotEventResult(_result)

        bot_event_log = cls(
            bot_id=bot_id,
            request_id=request_id,
            event=event,
            code=code,
            datetime_=datetime_,
            result=result,
        )

        bot_event_log.additional_properties = d
        return bot_event_log

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from io import BytesIO
from typing import Any, Dict, Optional, Union, cast

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...types import UNSET, File, Response, Unset


def _get_kwargs(
    file_id: str,
    *,
    dl: Union[Unset, int] = UNSET,
) -> Dict[str, Any]:
    params: Dict[str, Any] = {}

    params["dl"] = dl

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: Dict[str, Any] = {
        "method": "get",
        "url": f"/files/{file_id}",
        "params": params,
    }

    return _kwargs


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[Union[Any, File]]:
    if response.status_code == HTTPStatus.OK:
        response_200 = File(payload=BytesIO(response.content))

        return response_200
    if response.status_code == HTTPStatus.FORBIDDEN:
        response_403 = cast(Any, None)
        return response_403
    if response.status_code == HTTPStatus.NOT_FOUND:
        response_404 = cast(Any, None)
        return response_404
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[Union[Any, File]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    file_id: str,
    *,
    client: Union[AuthenticatedClient, Client],
    dl: Union[Unset, int] = UNSET,
) -> Response[Union[Any, File]]:
    """ファイルをダウンロード

     指定したファイル本体を取得します。
    指定したファイルへのアクセス権限が必要です。

    Args:
        file_id (str):
        dl (Union[Unset, int]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Any, File]]
    """

    kwargs = _get_kwargs(
        file_id=file_id,
        dl=dl,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    file_id: str,
    *,
    client: Union[AuthenticatedClient, Client],
    dl: Union[Unset, int] = UNSET,
) -> Optional[Union[Any, File]]:
    """ファイルをダウンロード

     指定したファイル本体を取得します。
    指定したファイルへのアクセス権限が必要です。

    Args:
        file_id (str):
        dl (Union[Unset, int]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Any, File]
    """

    return sync_detailed(
        file_id=file_id,
        client=client,
        dl=dl,
    ).parsed


async def asyncio_detailed(
    file_id: str,
    *,
    client: Union[AuthenticatedClient, Client],
    dl: Union[Unset, int] = UNSET,
) -> Response[Union[Any, File]]:
    """ファイルをダウンロード

     指定したファイル本体を取得します。
    指定したファイルへのアクセス権限が必要です。

    Args:
        file_id (str):
        dl (Union[Unset, int]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Any, File]]
    """

    kwargs = _get_kwargs(
        file_id=file_id,
        dl=dl,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    file_id: str,
    *,
    client: Union[AuthenticatedClient, Client],
    dl: Union[Unset, int] = UNSET,
) -> Optional[Union[Any, File]]:
    """ファイルをダウンロード

     指定したファイル本体を取得します。
    指定したファイルへのアクセス権限が必要です。

    Args:
        file_id (str):
        dl (Union[Unset, int]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Any, File]
    """

    return (
        await asyncio_detailed(
            file_id=file_id,
            client=client,
            dl=dl,
        )
    ).parsed
